@@ -5,7 +5,6 @@
 
 import os
 import subprocess
-from typing import Dict
 import asyncio
 import threading
 
@@ -14,8 +13,8 @@ import threading
 ENCODING = "gbk"
 
 # --------------------------#
-
-cache: Dict[str, object] = {}
+#
+# cache: Dict[str, object] = {}
 
 
 # def single_instance(cls: type):
@@ -75,16 +74,17 @@ class Aria2Server(metaclass=SingletonType):
         等待进程结束
         :return:
         """
-        self.process.wait()
+        code = self.process.wait()
         self._is_running = False
+        return code
 
     def terminate(self):
         self.process.terminate()
-        self.wait()
+        return self.wait()
 
     def kill(self):
         self.process.kill()
-        self.wait()
+        return self.wait()
 
     @property
     def pid(self):
@@ -113,24 +113,22 @@ class AsyncAria2Server(Aria2Server):
         super().__init__(*args, daemon=daemon)
 
     async def start(self):
-        cmd = ""
-        for i in self.cmd:
-            cmd += i + " "
-        # program, *args = self.cmd
-        self.process = await asyncio.create_subprocess_shell(cmd)
+        program, *args = self.cmd
+        self.process = await asyncio.create_subprocess_exec(program, *args)
         self._is_running = True
 
     async def wait(self):
-        await self.process.wait()
+        code = await self.process.wait()
         self._is_running = False
+        return code
 
     async def terminate(self):
         self.process.terminate()
-        await self.wait()
+        return await self.wait()
 
     async def kill(self):
         self.process.kill()
-        await self.wait()
+        return await self.wait()
 
     async def __aenter__(self):
         await self.start()
